# ------------------------------------------------------------------------------
#
#   Anton Synytsia
#   anton.synytsia@gmail.com
#
# ------------------------------------------------------------------------------

require 'ams_WindowSettings.rb'

msg = "You cannot load SU Window Settings extension until SketchUp meets all the plugin compatibility demands!"
raise(msg) unless defined?(AMS::Plugins::WindowSettings::NAME)

AMS::Lib.require 'sketchup_window.rb'
AMS::Lib.require 'keyboard.rb'
AMS::Lib.require 'screen.rb'
AMS::Lib.require 'system.rb'
AMS::Lib.require 'translate.rb'

module AMS::Plugins::WindowSettings

  @dir = File.dirname(__FILE__)
  @dlg = nil
  @hwnd = 0
  @initialized = false
  @borders = [] # [top, bottom, left, right, top+bottom, left+right]
  @borders_cleared = false
  @orig = {}
  @snap = 25 # snap distance in pixels
  @standard = {
    :version => VERSION,
    :fullscreen => false,
    :menubar => true,
    :viewCE => true,
    :container1 => true,
    :container2 => true,
    :container3 => true,
    :container4 => true,
    :statusbar => true,
    :scenesbar => true,
    :dialogs => true,
    :toolbars => true,
    :retain_on_start => false,
    :dlg_on_start => false,
    :transparent => false,
    :clear_borders => false,
    :open_sections => 'section1,section2',
    :snapped => [2,1], # [col, row]
    :first_time => true
  }
  @saved = @standard.dup
  TRANSLATE = AMS::Translate.new('commands', File.join(@dir, 'lang'))

  class << self

    private

    def su
      AMS::Sketchup
    end


    def wins
      AMS::Windows
    end


    def win
      AMS::Window
    end


    def panel_snap_to_view(wr)
      x = wr[0]
      y = wr[1]
      w = wr[2]-wr[0]
      h = wr[3]-wr[1]
      vr = AMS::Sketchup::Viewport.rect
      @saved[:snapped] = [0,0]
      if y.between?(vr[1]-h,vr[3])
        if (x-vr[0]).abs <= @snap
          x = vr[0]
          @saved[:snapped][0] = 1
        elsif (x+w-vr[2]).abs <= @snap
          x = vr[2]-w
          @saved[:snapped][0] = 2
        end
      end
      if x.between?(vr[0]-w,vr[2])
        if ((y-vr[1]).abs <= @snap)
          y = vr[1]
          @saved[:snapped][1] = 1
        elsif ((y+h-vr[3]).abs <= @snap)
          y = vr[3]-h
          @saved[:snapped][1] = 2
        end
      end
      return [x,y]
    end


    def panel_clip_to_screen(wr)
      res = AMS::Screen.resolution
      x = wr[0]
      y = wr[1]
      w = wr[2]-wr[0]
      h = wr[3]-wr[1]
      x = res[0]-w if (x+w > res[0])
      x = 0 if (x < 0)
      y = res[1]-h if (y+h > res[1])
      y = 0 if (y < 0)
      return [x,y]
    end

    public

    def load_defaults
      reg = Sketchup.read_default("Plugins_ams", "WindowSettings", "{}")
      begin
        opts = eval(reg)
        raise if !opts.is_a?(Hash)
        raise if opts.size != @standard.size
        raise if (opts[:version] != @standard[:version])
      rescue Exception => e
        opts = @standard.dup
      end
      @saved.merge!(opts)
      # Assign F12 shortcut to set/remove menu bar (if F12 not taken by some
      # other shortcut).
      if @saved[:first_time]
        shortcuts = su::Menubar.get_registry_shortcuts
        window = AMS::Sketchup::Menubar.get_menu_string_by_pos(6)
        path = "#{window}/#{TRANSLATE['Settings']}/#{TRANSLATE['Menu Bar']}"
        if shortcuts['F12'].nil? or (shortcuts['F12'] == path) or (shortcuts['F12'] =~ /Window\/Settings\/Menu Bar/i)
          reg = AMS::Sketchup::Registry
          num = reg.read('Settings/Num_Shortcuts')
          val = "0 0 0 F12 #{path}"
          found = false
          for i in 1..num
            v = reg.read("Settings/Shortcut_#{i}")
            next if v.nil? or ((v =~ /Window\/Settings\/Menu Bar/i).nil? and !v.include?(path))
            reg.write("Settings/Shortcut_#{i}", val)
            found = true
            break
          end
          if !found
            reg.write('Settings/Num_Shortcuts', num+1)
            reg.write("Settings/Shortcut_#{num+1}", val)
          end
        end
        @saved[:first_time] = false
      end
      opts
    end


    def save_defaults
      @saved[:menubar] = true if (@saved[:retain_on_start] and !@saved[:dlg_on_start])
      # First turn object into String, then escape and remove wrapping quotes.
      reg = @saved.inspect.inspect[1..-2]
      Sketchup.write_default("Plugins_ams", "WindowSettings", reg)
    end


    def get_settings
      settings = {
        :fullscreen => AMS::Sketchup::MainWindow.fullscreen?,
        :menubar => su::Menubar.set?,
        :viewCE => su::Viewport.client_edge_set?,
        :container1 => su::ToolbarContainers.visible?(1),
        :container2 => su::ToolbarContainers.visible?(2),
        :container3 => su::ToolbarContainers.visible?(3),
        :container4 => su::ToolbarContainers.visible?(4),
        :statusbar => su::Statusbar.visible?,
        :scenesbar => su::Scenesbar.visible?,
        :dialogs => (su.get_dialogs.zero? or su.get_visible_dialogs(false) > 0),
        :toolbars => (su.get_toolbars.zero? or su.get_visible_toolbars(false) > 0)
      }
      settings
    end


    def set_settings(s)
      r = []
      r.push su::ToolbarContainers.show(1, s[:container1], false)
      r.push su::ToolbarContainers.show(2, s[:container2], false)
      r.push su::ToolbarContainers.show(3, s[:container3], false)
      r.push su::ToolbarContainers.show(4, s[:container4], false)
      r.push su::Statusbar.show(s[:statusbar], false)
      r.push su::Scenesbar.show(s[:scenesbar], false)
      su::Viewport.set_client_edge(s[:viewCE])
      r1 = su::Menubar.set(s[:menubar])
      r2 = su::MainWindow.set_fullscreen(s[:fullscreen], 2)
      su.refresh if !r1 and !r2 and r.include?(true)
      su.show_toolbars(s[:toolbars])
      su.show_dialogs(s[:dialogs])
      true
    end


    def save
      @saved.merge!(get_settings)
    end


    def retain
      set_settings(@saved)
    end


    def reset
      set_settings(@standard)
    end


    def open_dialog
      if win.valid?(@hwnd)
        win.show(@hwnd, true)
        return false
      end
      # Create dialog
      title = 'SU Window Settings'
      width = 262
      height = 542
      offset = [0,0]
      res = AMS::Screen.resolution
      url = File.join(@dir, 'index.html')
      @dlg = UI::WebDialog.new(title, false, 'WindowSettings', width, height, res[0]-400, 130, false)
      # Callbacks
      @dlg.add_action_callback('init'){ |dlg, params|
        dialog_calc_borders
        cmd = "set_open_sections('#{@saved[:open_sections]}')"
        dlg.execute_script(cmd)
        unless @initialized
          @initialized = true
          dialog_set_transparent @saved[:transparent]
          dialog_clear_borders @saved[:clear_borders]
          su.add_observer(self)
        end
        dialog_update_snapped
        dialog_update_options
      }
      @dlg.set_on_close(){
        @dlg = nil
        @hwnd = 0
        @initialized = false
        @borders_cleared = false
        su.remove_observer(self)
      }
      @dlg.add_action_callback('adjust_size'){ |dlg, params|
        cx,cy = eval(params)
        win.set_size(@hwnd, cx+@borders[5], cy+@borders[4], false)
      }
      @dlg.add_action_callback('drag_start'){ |dlg, params|
        c = AMS::Cursor.get_pos
        d = win.origin(@hwnd)
        offset = [c[0]-d[0], c[1]-d[1]]
      }
      @dlg.add_action_callback('drag'){ |dlg, params|
        c = AMS::Cursor.get_pos
        x,y = [c[0]-offset[0], c[1]-offset[1]]
        w,h = win.size(@hwnd)
        if @borders_cleared
          wr = [x, y, x+w, y+h]
          x,y = panel_snap_to_view(wr)
        end
        wr = [x, y, x+w, y+h]
        x,y = panel_clip_to_screen(wr)
        win.set_pos(@hwnd, x, y, false)
      }
      @dlg.add_action_callback('drag_end'){ |dlg, params|
      }
      @dlg.add_action_callback('collapsible_changed'){ |dlg, params|
        @saved[:open_sections] = params
      }
      @dlg.add_action_callback('input_changed'){ |dlg, params|
        data = eval(params)
        case data[0]
        when :r1; su::MainWindow.set_fullscreen
        when :r2; su::MainWindow.maximize
        when :r3; su::MainWindow.minimize
        when :r4; su::MainWindow.restore
        when :cb1
          su::Menubar.set(data[1])
        when :cb2; su::Viewport.set_client_edge(data[1])
        when :cb3
          if su::ToolbarContainers.empty?(6)
            cmd = "document.getElementById('cb3').checked = false"
            dlg.execute_script(cmd)
          else
            su::ToolbarContainers.show(5, data[1])
          end
        when :cb4
          if su::Scenesbar.empty?
            cmd = "document.getElementById('cb4').checked = false"
            dlg.execute_script(cmd)
          else
            su::Scenesbar.show(data[1])
          end
        when :cb5; su::Statusbar.show(data[1])
        when :cb6; @saved[:retain_on_start] = data[1]
        when :cb7; @saved[:dlg_on_start] = data[1]
        when :cb8; dialog_set_transparent(data[1])
        when :cb9; dialog_clear_borders(data[1])
        end
        dialog_set_active
      }
      @dlg.add_action_callback('caption_changed'){ |dlg, params|
        params = '' if params.nil?
        su::MainWindow.caption = params
      }
      @dlg.add_action_callback('button_clicked'){ |dlg, params|
        case params.to_sym
        when :f_show; su.show_toolbars
        when :f_hide; su.show_toolbars false
        when :f_close; su.close_toolbars
        when :d_show; su.show_dialogs
        when :d_hide; su.show_dialogs false
        when :d_close; su.close_dialogs
        when :save; save
        when :retain; retain
        when :reset; reset
        end
      }
      # Set content
      @dlg.set_file(url)
      # Show dialog
      @dlg.show
      # Find a handle to the dialog window.
      @hwnd = su.window_by_caption(title)
      su.ignore_dialog(@hwnd)
      su.add_observer(self)
      # Modify window styles
      exstyle = wins::GetWindowLong.call(@hwnd, -20)
      style = wins::GetWindowLong.call(@hwnd, -16)
      os = AMS::System.windows_version
      ex_to_add = (os >= 6.0 ? 0x00080000 : 0) # ( WS_EX_LAYERED )
      ex_to_remove = 0x00000001 # ( WS_EX_DLGMODALFRAME )
      to_add = 0
      to_remove = 0x00010000 # ( WS_MAXIMIZEBOX )
      wins::LockWindowUpdate.call @hwnd
      wins::SetWindowLong.call( @hwnd, -20, (exstyle & ~ex_to_remove) | ex_to_add )
      wins::SetWindowLong.call( @hwnd, -16, (style & ~to_remove) | to_add )
      wins::LockWindowUpdate.call nil
      # Update set styles.
      wins::SetWindowPos.call(@hwnd, 0, 0,0, 0,0, 0x0267)
      # Modify window icon
      path = File.join(@dir, 'images/settings.ico')
      handle = wins::LoadImage.call(nil, path, 1, 0, 0, 0x00000050)
      # WM_SETICON
      #   http://msdn.microsoft.com/en-us/library/windows/desktop/ms632643(v=vs.85).aspx
      wins::SendMessage2.call(@hwnd, 0x0080, 0, handle)
      wins::SendMessage2.call(@hwnd, 0x0080, 1, handle)
      true
    end


    def close_dialog
      return false if win.invalid?(@hwnd)
      @dlg.close
      true
    end


    def dialog_open?
      win.valid?(@hwnd)
    end


    def dialog_set_active
      return false if win.invalid?(@hwnd)
      win.set_active(@hwnd)
      true
    end


    def dialog_calc_borders
      return false if win.invalid?(@hwnd)
      wr = win.rect(@hwnd)
      cr = win.client_rect(@hwnd)
      @borders[0] = cr[1]-wr[1] # top
      @borders[1] = wr[3]-cr[3] # bottom
      @borders[2] = cr[0]-wr[0] # left
      @borders[3] = wr[2]-cr[2] # right
      @borders[4] = @borders[0] + @borders[1] # top + bottom
      @borders[5] = @borders[2] + @borders[3] # left + right
      true
    end


    def dialog_adjust_size
      cmd = "setTimeout(adjust_size, 0);"
      @dlg.execute_script(cmd)
    end


    def dialog_snap_to_view
      return false if win.invalid?(@hwnd)
      wr = win.rect(@hwnd)
      x,y = panel_snap_to_view(wr)
      return false if ([x,y] == wr[0..1])
      win.set_pos(@hwnd, x, y, false)
      true
    end


    def dialog_clip_to_screen
      return false if win.invalid?(@hwnd)
      wr = win.rect(@hwnd)
      x,y = panel_clip_to_screen(wr)
      return false if ([x,y] == wr[0..1])
      win.set_pos(@hwnd, x, y, false)
      true
    end


    def dialog_update_snapped
      return false if win.invalid?(@hwnd)
      return false unless @borders_cleared
      wr = win.rect(@hwnd)
      vr = su::Viewport.rect
      x = wr[0]
      y = wr[1]
      w = wr[2]-wr[0]
      h = wr[3]-wr[1]
      # Check whether the dialog must be snapped if not already snapped.
      s = @saved[:snapped]
      if s.include?(0)
        temp = s.dup
        panel_snap_to_view(wr)
        @saved[:snapped] = temp if (@saved[:snapped] == [0,0])
        s = @saved[:snapped]
        if (s[0] != 0 and y.between?(vr[1]-h,vr[3]))
          x = (s[0] == 1 ? vr[0] : vr[2]-w)
        end
        if (s[1] != 0 and x.between?(vr[0]-w,vr[2]))
          y = (s[1] == 1 ? vr[1] : vr[3]-h)
        end
      else
        x = (s[0] == 1 ? vr[0] : vr[2]-w)
        y = (s[1] == 1 ? vr[1] : vr[3]-h)
      end
      x,y = panel_clip_to_screen([x, y, x+w, y+h])
      return false if ([x,y] == wr[0..1])
      win.set_pos(@hwnd, x, y, false)
      true
    end


    def dialog_set_opacity(v = 255)
      return 255 if win.invalid?(@hwnd)
      if (AMS::System.windows_version < 6.0)
        cmd = "document.getElementById('cb8').checked = false"
        @dlg.execute_script(cmd)
        return 255
      end
      if v.is_a?(Float)
        v *= 255 if v.bettween?(0,1)
        v.round!
      end
      v = 0 if (v < 0)
      v = 255 if (v > 255)
      wins::SetLayeredWindowAttributes.call(@hwnd, 0, v, 2)
      state = (v != 255)
      cmd = "document.getElementById('cb8').checked = #{state}"
      @dlg.execute_script(cmd)
      @saved[:transparent] = state
      v
    end


    def dialog_set_transparent(state = true)
      return false if win.invalid?(@hwnd)
      v = ( state == true ? 220 : 255)
      (dialog_set_opacity(v) != 255)
    end


    def dialog_clear_borders(state = true)
      return false if win.invalid?(@hwnd)
      state = ( state == true )
      return false if (@borders_cleared == state)
      wins::ShowWindow.call(@hwnd, 1) # make sure dialog is not minimized
      x,y = win.client_origin(@hwnd)
      if state
        @orig[:style_ex] = wins::GetWindowLong.call(@hwnd, -20)
        @orig[:style] = wins::GetWindowLong.call(@hwnd, -16)
        @orig[:borders] = @borders.dup
        os = AMS::System.windows_version
        layered = (os >= 6.0 ? 0x00080000 : 0) # ( WS_EX_LAYERED )
        style_ex = (0x00010000 | layered) # ( WS_EX_CONTROLPARENT | WS_EX_LAYERED )
        style = 0x94000000 # ( WS_POPUP | WS_VISIBLE | WS_CLIPSIBLINGS )
      else
        style_ex = @orig[:style_ex]
        style = @orig[:style]
        x -= @orig[:borders][2]
        y -= @orig[:borders][0]
      end
      win.set_pos(@hwnd, x, y, false)
      wins::LockWindowUpdate.call @hwnd
      wins::SetWindowLong.call(@hwnd, -20, style_ex)
      wins::SetWindowLong.call(@hwnd, -16, style)
      wins::LockWindowUpdate.call nil
      # Update set styles.
      wins::SetWindowPos.call(@hwnd, 0, 0,0, 0,0, 0x0267)
      dialog_calc_borders
      dialog_adjust_size
      @saved[:clear_borders] = state
      @borders_cleared = state
      cmd = "document.getElementById('cb9').checked = #{state}"
      @dlg.execute_script(cmd)
      true
    end


    def dialog_borders_cleared?
      return false if win.invalid?(@hwnd)
      @borders_cleared
    end


    def dialog_update_options
      return false unless @initialized
      opt = {}
      cmd = "document.getElementById('txt1').value = '#{su::MainWindow.caption}'"
      @dlg.execute_script(cmd)
      if su::MainWindow.fullscreen?
        id = :r1
      else
        id = su::MainWindow.maximized? ? :r2 : (su::MainWindow.minimized? ? :r3 : :r4)
      end
      opt[id] = true
      opt[:cb1] = su::Menubar.set?
      opt[:cb2] = su::Viewport.client_edge_set?
      opt[:cb3] = su::ToolbarContainers.visible?(5)
      opt[:cb4] = su::Scenesbar.visible?
      opt[:cb5] = su::Statusbar.visible?
      opt[:cb6] = @saved[:retain_on_start]
      opt[:cb7] = @saved[:dlg_on_start]
      opt[:cb8] = @saved[:transparent]
      opt[:cb9] = @saved[:clear_borders]
      opt.each{ |k,v|
        cmd = "document.getElementById('#{k}').checked = #{v}"
        @dlg.execute_script(cmd)
      }
      true
    end

    # Settings Observer

    def swo_mw_onCaptionChanged(text)
      return unless @initialized
      cmd = "document.getElementById('txt1').value = \"#{text}\""
      @dlg.execute_script(cmd)
    end


    def swo_mw_onSwitchFullscreen(state)
      return unless @initialized
      if state
        id = :r1
      else
        main = su::MainWindow
        id = main.maximized? ? :r2 : (main.minimized? ? :r3 : :r4)
      end
      cmd = "document.getElementById('#{id}').checked = true"
      @dlg.execute_script(cmd)
      dialog_update_snapped
    end


    def swo_mw_onMaximize
      return unless @initialized
      cmd = "document.getElementById('r2').checked = true"
      @dlg.execute_script(cmd)
      dialog_update_snapped
    end


    def swo_mw_onMinimize
      return unless @initialized
      cmd = "document.getElementById('r3').checked = true"
      @dlg.execute_script(cmd)
    end


    def swo_mw_onRestore
      return unless @initialized
      cmd = "document.getElementById('r4').checked = true"
      @dlg.execute_script(cmd)
      dialog_update_snapped
    end


    def swo_mb_onSetRemove(state)
      return unless @initialized
      cmd = "document.getElementById('cb1').checked = #{state}"
      @dlg.execute_script(cmd)
      dialog_update_snapped
    end


    def swo_mw_onActivate
      return unless @initialized
      main = su::MainWindow
      if main.fullscreen?
        id = :r1
      else
        id = main.maximized? ? :r2 : (main.minimized? ? :r3 : :r4)
      end
      cmd = "document.getElementById('#{id}').checked = true"
      @dlg.execute_script(cmd)
    end


    def swo_mw_onExitSizeMove
      dialog_update_snapped
    end


    def swo_vp_onSetRemoveClientEdge(state)
      return unless @initialized
      cmd = "document.getElementById('cb2').checked = #{state}"
      @dlg.execute_script(cmd)
      dialog_update_snapped
    end


    def swo_tbc_onVisibilityChanged(bar, state)
      return unless @initialized
      v = su::ToolbarContainers.visible?(5)
      cmd = "document.getElementById('cb3').checked = #{v}"
      @dlg.execute_script(cmd)
      dialog_update_snapped
    end


    def swo_tbc_onFilled(bar)
      swo_tbc_onVisibilityChanged(bar, nil)
    end


    def swo_tbc_onEmptied(bar)
      swo_tbc_onVisibilityChanged(bar, nil)
    end


    def swo_scb_onVisibilityChanged(state)
      return unless @initialized
      cmd = "document.getElementById('cb4').checked = #{state}"
      @dlg.execute_script(cmd)
      dialog_update_snapped
    end


    def swo_stb_onVisibilityChanged(state)
      return unless @initialized
      cmd = "document.getElementById('cb5').checked = #{state}"
      @dlg.execute_script(cmd)
      dialog_update_snapped
    end

  end # proxy class


  class MyAppObserver < Sketchup::AppObserver

    def onQuit
      ::AMS::Plugins::WindowSettings.save_defaults
    end

  end # class MyAppObserver


  unless file_loaded?(__FILE__)

    load_defaults
    Sketchup.add_observer(MyAppObserver.new)
    UI.start_timer(0.5, false){
        retain if @saved[:retain_on_start]
        open_dialog if @saved[:dlg_on_start]
    }
    toolbar = UI::Toolbar.new 'WS'

    submenu = UI.menu('Window').add_submenu(TRANSLATE["Settings"])

    cmd = UI::Command.new('cmd'){
      state = !su::MainWindow.fullscreen?
      su::MainWindow.set_fullscreen(state)
    }
    cmd.menu_text = cmd.tooltip = TRANSLATE["Full Screen"]
    cmd.status_bar_text = TRANSLATE["Toggle main window full screen."]
    cmd.set_validation_proc {
      su::MainWindow.fullscreen? ? MF_CHECKED : MF_UNCHECKED
    }
    cmd.small_icon = 'images/fullscreen16.png'
    cmd.large_icon = 'images/fullscreen24.png'
    toolbar.add_item(cmd)
    submenu.add_item(cmd)

    cmd = UI::Command.new('cmd'){
      state = !su::Menubar.set?
      su::Menubar.set state
    }
    cmd.menu_text = cmd.tooltip = TRANSLATE["Menu Bar"]
    cmd.status_bar_text = TRANSLATE["Set/remove menu bar."]
    cmd.set_validation_proc {
      su::Menubar.set? ? MF_CHECKED : MF_UNCHECKED
    }
    cmd.small_icon = 'images/menubar16.png'
    cmd.large_icon = 'images/menubar24.png'
    toolbar.add_item(cmd)
    submenu.add_item(cmd)

    cmd = UI::Command.new('cmd'){
      state = !su::Viewport.client_edge_set?
      su::Viewport.set_client_edge state
    }
    cmd.menu_text = TRANSLATE["View Border"]
    cmd.status_bar_text = TRANSLATE["Set/remove view client edge."]
    cmd.set_validation_proc {
      su::Viewport.client_edge_set? ? MF_CHECKED : MF_UNCHECKED
    }
    submenu.add_item(cmd)

    cmd = UI::Command.new('cmd'){
      state = !su::ToolbarContainers.visible?(5)
      su::ToolbarContainers.show(5, state)
    }
    cmd.menu_text = TRANSLATE["Toolbar Containers"]
    cmd.status_bar_text = TRANSLATE["Show/hide toolbar containers."]
    cmd.set_validation_proc {
      next MF_GRAYED if su::ToolbarContainers.empty?(6)
      su::ToolbarContainers.visible?(5) ? MF_CHECKED : MF_UNCHECKED
    }
    submenu.add_item(cmd)

    cmd = UI::Command.new('cmd'){
      state = !su::Scenesbar.visible?
      su::Scenesbar.show state
    }
    cmd.menu_text = TRANSLATE["Scenes Bar"]
    cmd.status_bar_text = TRANSLATE["Show/hide scenes bar."]
    cmd.set_validation_proc {
      next MF_GRAYED if su::Scenesbar.empty?
      su::Scenesbar.visible? ? MF_CHECKED : MF_UNCHECKED
    }
    submenu.add_item(cmd)

    cmd = UI::Command.new('cmd'){
      state = !su::Statusbar.visible?
      su::Statusbar.show state
    }
    cmd.menu_text = TRANSLATE["Status Bar"]
    cmd.status_bar_text = TRANSLATE["Show/hide status bar."]
    cmd.set_validation_proc {
      su::Statusbar.visible? ? MF_CHECKED : MF_UNCHECKED
    }
    submenu.add_item(cmd)

    submenu.add_separator

    cmd = UI::Command.new('cmd'){
      su.show_dialogs(su.get_visible_dialogs.zero?)
    }
    cmd.menu_text = TRANSLATE["Dialogs"]
    cmd.status_bar_text = TRANSLATE["Show/hide all dialogs, including the Ruby Console."]
    cmd.set_validation_proc {
      next MF_GRAYED if su.get_dialogs.zero?
      su.get_visible_dialogs(false) > 0 ? MF_CHECKED : MF_UNCHECKED
    }
    submenu.add_item(cmd)

    cmd = UI::Command.new('cmd'){ su.close_dialogs }
    cmd.menu_text = TRANSLATE["Close Dialogs"]
    cmd.status_bar_text = TRANSLATE["Close all dialogs, including the Ruby Console."]
    cmd.set_validation_proc {
      next MF_GRAYED if su.get_dialogs(false).zero?
    }
    submenu.add_item(cmd)

    cmd = UI::Command.new('cmd'){
      su.show_toolbars(su.get_visible_toolbars.zero?)
    }
    cmd.menu_text = TRANSLATE["Toolbars"]
    cmd.status_bar_text = TRANSLATE["Show/hide all floating toolbars."]
    cmd.set_validation_proc {
      next MF_GRAYED if su.get_toolbars.zero?
      su.get_visible_toolbars(false) > 0 ? MF_CHECKED : MF_UNCHECKED
    }
    submenu.add_item(cmd)

    cmd = UI::Command.new('cmd'){ su.close_toolbars }
    cmd.menu_text = TRANSLATE["Close Toolbars"]
    cmd.status_bar_text = TRANSLATE["Close all floating toolbars."]
    cmd.set_validation_proc {
      next MF_GRAYED if su.get_toolbars(false).zero?
    }
    submenu.add_item(cmd)

    submenu.add_separator

    cmd = UI::Command.new('cmd'){ save }
    cmd.menu_text = TRANSLATE["Save"]
    cmd.status_bar_text = TRANSLATE["Save current settings."]
    submenu.add_item(cmd)

    cmd = UI::Command.new('cmd'){ retain }
    cmd.menu_text = TRANSLATE["Retain"]
    cmd.status_bar_text = TRANSLATE["Set saved settings."]
    submenu.add_item(cmd)

    cmd = UI::Command.new('cmd'){ reset }
    cmd.menu_text = TRANSLATE["Reset"]
    cmd.status_bar_text = TRANSLATE["Set default layout."]
    submenu.add_item(cmd)

    submenu.add_separator

    cmd = UI::Command.new('cmd'){
      dialog_open? ? close_dialog : open_dialog
    }
    cmd.menu_text = cmd.tooltip = TRANSLATE["Settings Dialog"]
    cmd.status_bar_text = TRANSLATE["Open/close settings dialog."]
    cmd.set_validation_proc {
      dialog_open? ? MF_CHECKED : MF_UNCHECKED
    }
    cmd.small_icon = 'images/settings16.png'
    cmd.large_icon = 'images/settings24.png'
    toolbar.add_item(cmd)
    submenu.add_item(cmd)

    cmd = UI::Command.new('cmd'){ ::UI.openURL('http://sketchucation.com/forums/viewtopic.php?f=323&t=42926') }
    cmd.menu_text = TRANSLATE["Website"]
    cmd.status_bar_text = TRANSLATE["Open a web browser to the plugin's homepage."]
    submenu.add_item(cmd)

    toolbar.show

    file_loaded(__FILE__)
  end

end # module AMS::Plugins::WindowSettings
